#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'adions025@gmail.com'

from inspect import getmembers
from json import JSONEncoder, loads, dumps
from multiprocessing import Pool, cpu_count


class Encoder(JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'to_json'):
            return obj.to_json()
        elif hasattr(obj, '__dict__'):
            return {key: val for key, val in getmembers(obj)
                    if not any((key.startswith('_'), callable(val), key == 'metadata'))}
        elif hasattr(obj, 'isoformat'):
            return f"{obj.isoformat(timespec='milliseconds')}Z"
        elif type(obj).__name__ == 'ndarray':
            return obj.tolist()
        return super(Encoder, self).default(obj)


def jsonize(data, rel=False):
    result = []
    data = tuple(data) if isinstance(data, list) else (data,)
    for entity in data:
        if hasattr(entity, '__table__'):
            columns = [x.name for x in entity.__table__.columns]
            relations = [x.key for x in entity.__mapper__.relationships]
            obj = {}
            for col in columns:
                attr = getattr(entity, col)
                obj[col] = (f"{attr.isoformat(timespec='milliseconds')}Z" if hasattr(attr, 'isoformat') else attr)
            for relation in relations:
                obj[relation] = jsonize(getattr(entity, relation), True)
            result.append(obj)
        elif not rel:
            result.append(loads(dumps(entity, cls=Encoder)))
    return result


def make_response(result):
    limit = 24_000_000
    content, result.content = result.content, None
    response = loads(dumps(result, cls=Encoder))
    if isinstance(content, list):
        if len(str(content)) > limit:
            with Pool(cpu_count()) as pool:
                response['content'] = pool.map(jsonize, content)
            pool.join()
        else:
            response['content'] = jsonize(content)
    else:
        response['content'] = jsonize(content)[0]
    return response



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


__author__ = 'adions025@gmail.com'

import os
from functools import wraps
from inspect import iscoroutinefunction, getfile
from typing import Any, Callable, TypeVar, cast

from restutil.result import ResData
from restutil.result import ResOperationType as resType


class Decorest:

    Fn = TypeVar('Fn', bound=Callable[..., Any])

    def __init__(self):
        pass

    def try_log(self, func: Fn):
        if iscoroutinefunction(func):
            @wraps(func)
            async def wrapper(*args, **kwargs):
                try:
                    return await func(*args, **kwargs)
                except Exception as exc:
                    return self.raise_exception(func, exc)
            return cast(self.Fn, wrapper)
        else:
            @wraps(func)
            def wrapper(*args, **kwargs):
                try:
                    return func(*args, **kwargs)
                except Exception as exc:
                    return self.raise_exception(func, exc)
            return wrapper

    def raise_exception(self, func, exc, result: ResData = None) -> ResData:
        if not result:
            result = ResData()
        result.add_result(message=f'Error occurred in {func.__name__}: {str(exc)} in {get_filename(func)}',
                          res_type=resType.EXCEPTION)
        return result


def get_filename(func):
    if getfile(func):
        return os.path.split(getfile(func))[-1]
    return ''



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'adions025@gmail.com'


class DictReader(object):

    def __init__(self, keys_dic=None, config_file=()):
        self.__config_file = config_file
        if keys_dic is None:
            self.__keys = ()
        else:
            self.__keys = keys_dic
        self.__result = None

    def __getattr__(self, attribute):
        try:
            result = DictReader(keys_dic=self.__keys + (attribute,), config_file=self.__config_file)
        except Exception as ex:
            raise ValueError(str(ex))
        return result

    def get(self, value_dict=None):
        try:
            if value_dict is None:
                dictionary_search = self.__config_file
            else:
                dictionary_search = value_dict
            for k in self.__keys:
                value = dictionary_search.get(k)
                if value is not None:
                    if isinstance(value, dict):
                        self.get(value_dict=value)
                    else:
                        if self.__result is None:
                            self.__result = value
        except Exception as ex:
            raise ValueError(str(ex))
        return self.__result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'adions025@gmail.com'

from enum import Enum


class ResOperation(object):
    def __init__(self, message, res_type):
        self.__message = message
        self.__result_type = res_type

    @property
    def message(self):
        return self.__message

    @property
    def result_type(self):
        return self.__result_type


class ResOperationType(Enum):
    ERROR = 0
    INFO = 1
    WARNING = 2
    SUCCESS = 3
    EXCEPTION = 4



#! usr/bin/python
# -*- coding: utf-8 *-*

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup

from pathlib import Path

root_dir = Path(__file__).parent
long_description = (root_dir / "README.md").read_text()

setup(
    name='restutil',
    packages=['restutil'],
    version='1.0.9',
    description='Rest Util Tools',
    author='Adonis Gonzalez Godoy',
    author_email='adions025@gmail.com',
    url='https://github.com/adions025/restutil',
    download_url='https://github.com/adions025/restutil',
    keywords=['Restful', 'Rest', 'Util', 'Tools'],
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    long_description=long_description,
    long_description_content_type='text/markdown'
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'adions025@gmail.com'

from typing import List, Any
from restutil.operations import ResOperation

from restutil.operations import ResOperationType


class ResData(object):

    def __init__(self):
        self.__content: Any = None
        self.__has_errors: bool = False
        self.__result_operations: List = []

    @property
    def result_operations(self) -> List:
        return self.__result_operations

    def add_result(self, message: str, res_type: ResOperationType) -> None:
        try:
            self.__result_operations.append(ResOperation(message, res_type))
            if res_type == ResOperationType.ERROR or res_type == ResOperationType.EXCEPTION:
                self.__has_errors = True
        except Exception as ex:
            raise ValueError(str(ex))

    def add_result_range(self, result_range: list) -> None:
        for res in result_range:
            self.add_result(res.message, res.result_type)

    def format_result_operations(self):
        message = ""
        for x in self.__result_operations:
            message = '%s %s - %s,' % (message, x.result_type.name, x.message)
        return message

    @property
    def content(self):
        return self.__content

    @content.setter
    def content(self, value):
        self.__content = value

    @property
    def has_errors(self):
        return self.__has_errors
